@@ -67,6 +67,29 @@ setGeneric(name = "colSums", useAsDefault = base::colSums, package="pbdDMAT")
 setGeneric(name = "rowMeans", useAsDefault = base::rowMeans, package="pbdDMAT")
 setGeneric(name = "colMeans", useAsDefault = base::colMeans, package="pbdDMAT")
 
+setGeneric(name="rowMin", 
+  function(x, ...)
+    standardGeneric("rowMin"),
+  package="pbdDMAT"
+)
+
+setGeneric(name="rowMax", 
+  function(x, ...)
+    standardGeneric("rowMax"),
+  package="pbdDMAT"
+)
+
+setGeneric(name="colMin", 
+  function(x, ...)
+    standardGeneric("colMin"),
+  package="pbdDMAT"
+)
+
+setGeneric(name="colMax", 
+  function(x, ...)
+    standardGeneric("colMax"),
+  package="pbdDMAT"
+)
 
 
 # Algebra
@@ -186,6 +209,12 @@ setGeneric(name="ldim",
   package="pbdDMAT"
 )
 
+setGeneric(name="llen", 
+  function(x, ...) 
+    standardGeneric("llen"), 
+  package="pbdDMAT"
+)
+
 setGeneric(name="bldim", 
   function(x, ...) 
     standardGeneric("bldim"), 
